/* $Id: nbp.h,v 3.1 1997/11/28 19:45:28 tom Exp $ */

/*
 * Copyright (c) 1997-2004, Gerben Wierda, R&A All rights reserved
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * Neither the name of R&A nor the names of its contributors may be used to
 * endorse or promote products derived from this software without specific prior
 * written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/* No Warranty of any kind */

#if !defined( _RnA_NBP_NOPOSIX_DEFINED)
#      define _RnA_NBP_NOPOSIX_DEFINED 1

#ifndef _RnA_NBP_INLINE 
#define _RnA_NBP_INLINE	static inline
#endif

#include <stdarg.h>
#include <sys/stat.h>

/* Force definition of SYS_getposix, SYS_setposix in <syscall.h> */
#if !defined( _POSIX_SOURCE)
#   define _POSIX_SOURCE 1
#   define _RnA_NBP_POSIX_SOURCE_OVERRIDE	1
#endif

#include <syscall.h>

/* Restore previous _POSIX_SOURCE situation. */
#if defined( _RnA_NBP_POSIX_SOURCE_OVERRIDE)
#   undef _POSIX_SOURCE
#   undef _RnA_NBP_POSIX_SOURCE_OVERRIDE
#endif

#define _RnA_NBP_getposix()	syscall( SYS_getposix)
#define _RnA_NBP_setposix(i)	syscall( SYS_setposix, (i))

/* XXX GCC (2.5.8) cannot handle inline functions with ... varargs. */
_RnA_NBP_INLINE int _RnA_noposix_open( const char *path, int flags, int mode)
{
    int result, prevstatus;
    if ((prevstatus = _RnA_NBP_getposix()) != 0) _RnA_NBP_setposix( 0);
    result = open( path, flags, mode);
    if (prevstatus != 0) _RnA_NBP_setposix( prevstatus);
    return result;
}
#undef open
#define open _RnA_noposix_open

_RnA_NBP_INLINE int _RnA_noposix_close( int fd)
{
    int result, prevstatus;
    if ((prevstatus = _RnA_NBP_getposix()) != 0) _RnA_NBP_setposix( 0);
    result = close( fd);
    if (prevstatus != 0) _RnA_NBP_setposix( prevstatus);
    return result;
}
#undef close
#define close _RnA_noposix_close

_RnA_NBP_INLINE int _RnA_noposix_write( int fd, const void *buf, int nbytes)
{
    int result, prevstatus;
    if ((prevstatus = _RnA_NBP_getposix()) != 0) _RnA_NBP_setposix( 0);
    result = write( fd, buf, nbytes);
    if (prevstatus != 0) _RnA_NBP_setposix( prevstatus);
    return result;
}
#undef write
#define write _RnA_noposix_write

_RnA_NBP_INLINE int _RnA_noposix_read( int fd, void *buf, int nbytes)
{
    int result, prevstatus;
    if ((prevstatus = _RnA_NBP_getposix()) != 0) _RnA_NBP_setposix( 0);
    result = read( fd, buf, nbytes);
    if (prevstatus != 0) _RnA_NBP_setposix( prevstatus);
    return result;
}
#undef read
#define read _RnA_noposix_read

_RnA_NBP_INLINE off_t _RnA_noposix_lseek( int fd, off_t offset, int whence)
{
    off_t result;
    int prevstatus;
    if ((prevstatus = _RnA_NBP_getposix()) != 0) _RnA_NBP_setposix( 0);
    result = lseek( fd, offset, whence);
    if (prevstatus != 0) _RnA_NBP_setposix( prevstatus);
    return result;
}
#undef lseek
#define lseek _RnA_noposix_lseek

_RnA_NBP_INLINE int _RnA_noposix_creat( char *path, int mode)
{
    int result, prevstatus;
    if ((prevstatus = _RnA_NBP_getposix()) != 0) _RnA_NBP_setposix( 0);
    result = creat( path, mode);
    if (prevstatus != 0) _RnA_NBP_setposix( prevstatus);
    return result;
}
#undef creat
#define creat _RnA_noposix_creat

extern void *		mmap( void *addr, size_t size, int mode, int options,
			      int fd, off_t offset );

_RnA_NBP_INLINE void *_RnA_noposix_mmap( void *addr, size_t size, int mode,
				       int options, int fd, off_t offset )
{
    void *result;
    int prevstatus;
    if ((prevstatus = _RnA_NBP_getposix()) != 0) _RnA_NBP_setposix( 0);
    result = mmap( addr, size, mode, options, fd, offset);
    if (prevstatus != 0) _RnA_NBP_setposix( prevstatus);
    return result;
}
#undef mmap
#define mmap _RnA_noposix_mmap

_RnA_NBP_INLINE int _RnA_noposix_munmap( void *addr, size_t size)
{
    int result, prevstatus;
    if ((prevstatus = _RnA_NBP_getposix()) != 0) _RnA_NBP_setposix( 0);
    result = syscall( SYS_munmap, addr, size);
    if (prevstatus != 0) _RnA_NBP_setposix( prevstatus);
    return result;
}
#undef munmap
#define munmap _RnA_noposix_munmap

#undef _RnA_NBP_getposix
#undef _RnA_NBP_setposix

#endif // _RnA_NBP_NOPOSIX_DEFINED
